// Yet Another Units Library (YANL) - A C++ library for zero-overhead
// dimensional analysis and unit manipulation/conversion.
// 
// Copyright (C) 2003 Matthias Christian Schabel
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#ifndef MCS_CGS_UNITS_HPP
#define MCS_CGS_UNITS_HPP

#include <string>

#include <mcs/units/dimensioned_quantity.hpp>
#include <mcs/units/physical_units.hpp>

namespace mcs {

namespace units { 

// CGS units
struct CGSModel 
{ 
	typedef PhysicalUnitRepresentation			                    representation_type;
	
	typedef DimensionedUnit<CGSModel,length_type>					length_unit;
	typedef DimensionedUnit<CGSModel,mass_type>						mass_unit;
	typedef DimensionedUnit<CGSModel,time_type>						time_unit;
	typedef DimensionedUnit<CGSModel,current_type>					current_unit;
	typedef DimensionedUnit<CGSModel,temperature_type>				temperature_unit;
	typedef DimensionedUnit<CGSModel,quantity_type>					quantity_unit;
	typedef DimensionedUnit<CGSModel,intensity_type>				intensity_unit;
	typedef DimensionedUnit<CGSModel,angle_type>					angle_unit;
	typedef DimensionedUnit<CGSModel,solid_angle_type>				solid_angle_unit;

	typedef DimensionedUnit<CGSModel,acceleration_type>				acceleration_unit;
	typedef DimensionedUnit<CGSModel,area_type>						area_unit;
	typedef DimensionedUnit<CGSModel,energy_type>					energy_unit;
	typedef DimensionedUnit<CGSModel,force_type>					force_unit;
	typedef DimensionedUnit<CGSModel,frequency_type>				frequency_unit;
	typedef DimensionedUnit<CGSModel,mass_density_type>				mass_density_unit;
	typedef DimensionedUnit<CGSModel,momentum_type>					momentum_unit;
	typedef DimensionedUnit<CGSModel,power_type>					power_unit;
	typedef DimensionedUnit<CGSModel,pressure_type>					pressure_unit;
	typedef DimensionedUnit<CGSModel,velocity_type>					velocity_unit;
	typedef DimensionedUnit<CGSModel,volume_type>					volume_unit;
	typedef DimensionedUnit<CGSModel,wavenumber_type>				wavenumber_unit;

	typedef DimensionedUnit<CGSModel,capacitance_type>				capacitance_unit;
	typedef DimensionedUnit<CGSModel,conductance_type>				conductance_unit;
	typedef DimensionedUnit<CGSModel,electric_charge_type>			electric_charge_unit;
	typedef DimensionedUnit<CGSModel,electric_potential_type>		electric_potential_unit;
	typedef DimensionedUnit<CGSModel,magnetic_flux_type>			magnetic_flux_unit;
	typedef DimensionedUnit<CGSModel,magnetic_flux_density_type>	magnetic_flux_density_unit;
	typedef DimensionedUnit<CGSModel,inductance_type>				inductance_unit;
	typedef DimensionedUnit<CGSModel,permittivity_type>				permittivity_unit;
	typedef DimensionedUnit<CGSModel,resistance_type>				resistance_unit;

	typedef DimensionedUnit<CGSModel,catalytic_activity_type>		catalytic_activity_unit;
	
	template<typename T> struct unit;
};

template<> 
struct CGSModel::unit<length_tag> 
{ 
	static std::string symbol()				{ return "centimeter"; }
	static std::string abbreviation()		{ return "cm"; }
	
	template<typename Y> static Y toBase()	{ return Y(0.01); }
};
	
template<> 
struct CGSModel::unit<mass_tag> 
{ 
	static std::string symbol()				{ return "gram"; }
	static std::string abbreviation()		{ return "g"; }
	
	template<typename Y> static Y toBase()	{ return Y(0.001); }
};
	
template<> 
struct CGSModel::unit<time_tag> 
{ 
	static std::string symbol()				{ return "second"; }
	static std::string abbreviation()		{ return "s"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct CGSModel::unit<current_tag> 
{ 
	static std::string symbol()				{ return "biot"; }
	static std::string abbreviation()		{ return "Bi"; }
	
	template<typename Y> static Y toBase()	{ return Y(10); }
};
	
template<> 
struct CGSModel::unit<temperature_tag> 
{ 
	static std::string symbol()				{ return "kelvin"; }
	static std::string abbreviation()		{ return "K"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct CGSModel::unit<quantity_tag> 
{ 
	static std::string symbol()				{ return "mole"; }
	static std::string abbreviation()		{ return "mol"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct CGSModel::unit<intensity_tag> 
{ 
	static std::string symbol()				{ return "candela"; }
	static std::string abbreviation()		{ return "cd"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};

template<> 
struct CGSModel::unit<angle_tag> 
{ 
	static std::string symbol()				{ return "radian"; }
	static std::string abbreviation()		{ return "rd"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct CGSModel::unit<solid_angle_tag> 
{ 
	static std::string symbol()				{ return "steradian"; }
	static std::string abbreviation()		{ return "sr"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
namespace cgs {

static const CGSModel::length_unit					_centimeter,
													_centimeters;
static const CGSModel::mass_unit					_gram,
													_grams;
static const CGSModel::time_unit					_second,
													_seconds;
static const CGSModel::current_unit					_biot,
													_biots;
static const CGSModel::temperature_unit				_kelvin,
													_kelvins;
static const CGSModel::quantity_unit				_mole,
													_moles;
static const CGSModel::intensity_unit				_candela,
													_candelas;
static const CGSModel::angle_unit					_radian,
													_radians;
static const CGSModel::solid_angle_unit				_steradian,
													_steradians;													
												
static const CGSModel::acceleration_unit			_gal,
													_gals;
static const CGSModel::area_unit					_square_centimeter,
													_square_centimeters;
static const CGSModel::energy_unit					_erg,
													_ergs;
static const CGSModel::force_unit					_dyne,
													_dynes;
static const CGSModel::frequency_unit				_hertz;
static const CGSModel::pressure_unit				_barye,
													_baryes;
static const CGSModel::velocity_unit				_centimeter_per_second,
													_centimeters_per_second;
static const CGSModel::volume_unit					_cubic_centimeter,
													_cubic_centimeters;
static const CGSModel::wavenumber_unit				_kayser,
													_kaysers;
														
static const CGSModel::capacitance_unit				_abfarad,
													_abfarads;
static const CGSModel::conductance_unit				_abmho,
													_abmhos;
static const CGSModel::electric_charge_unit			_abcoulomb,
													_abcoulombs;
static const CGSModel::electric_potential_unit		_abvolt,
													_abvolts;
static const CGSModel::magnetic_flux_unit			_maxwell,
													_maxwells;
static const CGSModel::magnetic_flux_density_unit	_gauss;
static const CGSModel::inductance_unit				_abhenry,
													_abhenrys;
static const CGSModel::resistance_unit				_abohm,
													_abohms;

} // namespace cgs

template<class Y>
struct CGS : 
	public PhysicalUnitModel<Y,CGSModel>
{ };

} // namespace units

} // namespace mcs

#endif // MCS_CGS_UNITS_HPP