// Yet Another Units Library (YANL) - A C++ library for zero-overhead
// dimensional analysis and unit manipulation/conversion.
// 
// Copyright (C) 2003 Matthias Christian Schabel
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#ifndef MCS_FSF_UNITS_HPP
#define MCS_FSF_UNITS_HPP

#include <string>

#include <mcs/units/dimensioned_quantity.hpp>
#include <mcs/units/physical_units.hpp>

namespace mcs {

namespace units { 

// furlong/stone/fortnight units
//
//    purely for pedagogical value, demonstrating how to set up a new system of units
struct FSFModel 
{ 
    typedef PhysicalUnitRepresentation                                representation_type;
    
    typedef DimensionedUnit<FSFModel,length_type>                     length_unit;
    typedef DimensionedUnit<FSFModel,mass_type>                       mass_unit;
    typedef DimensionedUnit<FSFModel,time_type>                       time_unit;

    typedef DimensionedUnit<FSFModel,acceleration_type>               acceleration_unit;
    typedef DimensionedUnit<FSFModel,area_type>                       area_unit;
    typedef DimensionedUnit<FSFModel,energy_type>                     energy_unit;
    typedef DimensionedUnit<FSFModel,force_type>                      force_unit;
    typedef DimensionedUnit<FSFModel,frequency_type>                  frequency_unit;
    typedef DimensionedUnit<FSFModel,mass_density_type>               mass_density_unit;
    typedef DimensionedUnit<FSFModel,momentum_type>                   momentum_unit;
    typedef DimensionedUnit<FSFModel,power_type>                      power_unit;
    typedef DimensionedUnit<FSFModel,pressure_type>                   pressure_unit;
    typedef DimensionedUnit<FSFModel,velocity_type>                   velocity_unit;
    typedef DimensionedUnit<FSFModel,volume_type>                     volume_unit;
    typedef DimensionedUnit<FSFModel,wavenumber_type>                 wavenumber_unit;
    
    template<typename T> struct unit;
};

template<> 
struct FSFModel::unit<length_tag> 
{ 
	static std::string symbol()				{ return "furlong"; }
	static std::string abbreviation()		{ return "fl"; }
	
	template<typename Y> static Y toBase()	{ return Y(603.504); }
};
	
template<> 
struct FSFModel::unit<mass_tag> 
{ 
	static std::string symbol()				{ return "stone"; }
	static std::string abbreviation()		{ return "st"; }
	
	template<typename Y> static Y toBase()	{ return Y(6.363636); }
};
	
template<> 
struct FSFModel::unit<time_tag> 
{ 
	static std::string symbol()				{ return "fortnight"; }
	static std::string abbreviation()		{ return "fn"; }
	
	template<typename Y> static Y toBase()	{ return Y(1209600); }
};
	
namespace fsf {

static const FSFModel::length_unit          _furlong,
                                            _furlongs;
static const FSFModel::mass_unit            _stone;
static const FSFModel::time_unit            _fortnight,
                                            _fortnights;
                                                    
} // namespace fsf

template<class Y>
struct FSF
{
    typedef Y    value_type;
    
    typedef DimensionedQuantity<Y,typename FSFModel::length_unit>               Length;
    typedef DimensionedQuantity<Y,typename FSFModel::mass_unit>                 Mass;
    typedef DimensionedQuantity<Y,typename FSFModel::time_unit>                 Time;
};

} // namespace units

} // namespace mcs

#endif // MCS_FSF_UNITS_HPP